#include <TMatrixD.h>
#include <TMatrixDUtils.h>
#include <TVectorD.h>
#include <TArrayD.h>
#include <TSystem.h>
#include <TInverse.h>

#ifndef __CINT__
#include <TROOT.h>
#include <Rtypes.h>
#endif

class FillVectorD : public TElementPosActionD
{
public:
  FillVectorD(TArrayD &a)
    { fArray=a.GetArray(); }
  Double_t *fArray;
  inline void Operation(Double_t &element) { element = fArray[fI]; }
};

class ExtractVectorD : public TElementPosActionD
{
  Double_t *fArray;
public:
  ExtractVectorD(const TVectorD &v, Double_t *a)
    { fArray=a; }
  inline void Operation(Double_t &element) { fArray[fI] = element; }
};

class FillMatrixD : public TElementPosActionD
{
  Int_t    fnrCols;
  Double_t *fArray;
  inline void Operation(Double_t &element) { element = fArray[fI*fnrCols+fJ]; }
public:
  FillMatrixD(const TMatrixD &m, TArrayD &a) :
    fnrCols(m.GetNcols()) { fArray=a.GetArray(); }
};

class ExtractMatrixD : public TElementPosActionD
{
  Int_t     fno_cols;
  Double_t *fArray;
  void Operation(Double_t &element)
      { fArray[fI*fno_cols+fJ] = element; }
public:
  ExtractMatrixD(const TMatrixD &m,Double_t *a) :
    fno_cols(m.GetNcols()) { fArray=a; }
};

class TransPoseMatrixD : public TElementPosActionD
{
  Int_t    fnrRows;
  Double_t *fArray;
  inline void Operation(Double_t &element) { element = fArray[fJ*fnrRows+fI]; }
public:
  TransPoseMatrixD(TMatrixD &m,TArrayD &a) :
    fnrRows(m.GetNrows()) { fArray=a.GetArray(); }
};

class MultMatrixD : public TElementPosActionD
{
  Int_t    fnrRows,fnrCols;
  Double_t *m;
  inline void Operation(Double_t &element)
  {
    element = 0.0;
    for (Int_t k = 0; k < fnrRows; k++)
    {
      Int_t off = k*fnrCols;
      element += m[off+fI]*m[off+fJ];
    }
  }

public:
  MultMatrixD(TArrayD &a,Int_t nrR,Int_t nrC) :
    fnrRows(nrR),fnrCols(nrC) { m=a.GetArray(); }
};

void NormalEq(const Int_t nrPnts,const Int_t nrVar,TArrayD &a,TArrayD &c)
{
  Int_t nrCols = nrVar;
  Int_t nrRows = a.GetSize()/nrCols;

  // hessian (A^T A)
  TMatrixD ataInv(nrVar,nrVar);
  {
    TMatrixD hessian(nrVar,nrVar);
    MultMatrixD mult(a,nrRows,nrCols);
    hessian.Apply(mult);
    Double_t det;
    ataInv = hessian;
    ataInv.Invert(&det);
  }

  // gradient (A^T c)
  TVectorD gradient(nrPnts);
  {
    TMatrixD at(nrCols,nrRows);
    {
      TransPoseMatrixD tr(at,a);
      at.Apply(tr);
    }

    FillVectorD f(c);
    gradient.Apply(f);
    gradient *= at;
    TMatrixD pseudoInv(ataInv,TMatrixD::kMult,at);
    pseudoInv.Print();
  }

  // solution = (A^T A)^-1 A^T b
  TVectorD solution = gradient;
  solution *= ataInv;
  solution.Print();
}

#ifdef __CINT__

// This part is used in the interpreter, e.g.:
// root
// .L solve.C
// solve(..,..,.......)

void solve()
{
  gROOT->Reset();
  gSystem->Load("libMatrix.so");
  gSystem->Load("libInverse.so");

#endif
#ifndef __CINT__

// This is used when being compiled

int main(Int_t argc, Char_t **argv)
{
  TROOT solve("solve","solve linear fit y = c_0 + c_1 x");

#endif

  Int_t nrVar  = 2;
  Int_t nrPnts = 4;
  TArrayD x(nrPnts); TArrayD y(nrPnts); TArrayD e(nrPnts);

  x[0] = 0.0; y[0] = 1.4; e[0] = 0.5;
  x[1] = 1.0; y[1] = 1.5; e[1] = 0.2;
  x[2] = 2.0; y[2] = 3.7; e[2] = 1.0;
  x[3] = 3.0; y[3] = 4.1; e[3] = 0.5;

  TArrayD b(nrPnts);
  TArrayD a(nrPnts*nrVar);
  for (Int_t irow = 0; irow < nrPnts; irow++)
  {
    b[irow] = y[irow]/e[irow]; 
    for (Int_t icol = 0; icol < nrVar; icol++)
    {
      Int_t i = irow*nrVar+icol;
      if (icol == 0) a[i] = 1/e[irow]; 
      else           a[i] = x[irow]/e[irow];
    }
  }

  //solve through Normal Equations
  // calculate also the pseudo inverse brute force in order
  // to compare
  NormalEq(nrPnts,nrVar,a,b);

  printf("\nNow start using single value decomposition\n");
  // Calculate pseudo inverse
  {
    Int_t nrSingular;
    Double_t frac = 0.0;
    TArrayD tmp(a);
    TInverse::PseudoInv(tmp,nrPnts,nrVar,frac,nrSingular);
    for (Int_t i = 0; i < nrVar; i++)
    {
      for (Int_t j = 0; j < nrPnts; j++)
        printf("a[%d][%d] = %.4f ",i,j,tmp[i*nrPnts+j]);
      printf("\n");
    }
  }

  // Calculate solution using single value decomposition
  {
    TArrayD solution;
    TArrayD residual;
    const Double_t frac = 0.0;
    Int_t nrSingular = 0.0;
    TInverse::SvDeComp(a,b,solution,residual,nrPnts,nrVar,frac,nrSingular);
    for (Int_t i = 0; i < nrVar; i++)
      printf("%d: %.4f\n",i,solution[i]);
    printf("nrSingular: %d\n",nrSingular);
    printf("chi2      : %.4f\n",residual[0]);
  }

#ifndef __CINT__
  return 0;
#endif
}
